#!/bin/sh

if [ -z "${__included_boot_funcs-}" ]; then
__included_boot_funcs=1

BOOTNAMES="NAND SD1 SD2"
ROOT_NAND_NUM=7

NANDDEV="${NANDDEV:-/dev/mtdblock}"
SDDEV="${SDDEV:-/dev/mmcblk0p}"
SDROOT_LABEL_PFX="root-"
SDHOME_LABEL="home"
SDCONF_LABEL="conf"
SDSTORAGE_LABEL="storage"
SDROOT_OFFS=1 # 1 is uboot on FAT (try by labels first, however)

is_bootname() {
    echo "$BOOTNAMES" | grep -qw "$1"
}

get_root_device()
{
    local name="$1"

    if ! is_bootname "$name"; then
        echo "Unknown boot device: $1" >&2
        return 1
    fi

    case "$name" in
        NAND)
            echo "/dev/mtdblock$ROOT_NAND_NUM"
            ;;
        SD*)
            if [ -f "/dev/disk/by-label/${SDROOT_LABEL_PFX}${name#SD}" ]; then
                readlink -f "/dev/disk/by-label/${SDROOT_LABEL_PFX}${name#SD}"
            else
                echo "$SDDEV$((SDROOT_OFFS + ${name#SD}))"
            fi
            ;;
        *)
            echo "Unknown boot device: $1" >&2
            return 1
            ;;
    esac
}

get_device_name() {
    local dev="$1"

    if [ "${dev#$NANDDEV}" != "$dev" ]; then
        echo 'NAND'
    elif [ "${dev#$SDDEV}" != "$dev" ]; then
        if [ -d /dev/disk/by-label ]; then
            find /dev/disk/by-label -name "$SDROOT_LABEL_PFX*" | (
                while read label; do
                    if [ "$dev" = "$(readlink -f "$label")" ]; then
                        label="${label##*/}"
                        echo "SD${label#$SDROOT_LABEL_PFX}"
                        exit 0
                    fi
                done
                echo "SD$((${dev#$SDDEV} - SDROOT_OFFS))"
            )
        else
            echo "SD$((${dev#$SDDEV} - SDROOT_OFFS))"
        fi
    else
        echo 'UNKNOWN'
        return 1 
    fi
}

is_nand_device() {
    [ "$(get_device_name "${1:-}")" = 'NAND' ]
}

is_sd_device() {
    case "$(get_device_name "${1:-}")" in
        SD*)
            return 0
            ;;
    esac

    return 1
}

get_current_boot() {
  if  cat /proc/cmdline | grep -q ubi;  then
   echo 'NAND'
  elif  cat /proc/cmdline | grep -q "$SDDEV";  then
    # TODO: parse the number
    if cat /proc/cmdline | grep -q "$SDDEV$((SDROOT_OFFS + 1))";  then
      get_device_name "$SDDEV$((SDROOT_OFFS + 1))"
    elif  cat /proc/cmdline | grep -q "$SDDEV$((SDROOT_OFFS + 2))";  then
      get_device_name "$SDDEV$((SDROOT_OFFS + 2))"
    else
      echo 'ERROR: Unexpected MMC boot device' >&2
      return 2
    fi
  else
    echo 'UNKNOWN'
  fi
}

get_next_boot() {
  if fw_printenv bootcmd 2> /dev/null | grep -q "mmcboot";  then
    local mmcpart="$(fw_printenv -n mmcpart 2>/dev/null)"
    get_device_name "$SDDEV$mmcpart"
  elif fw_printenv bootcmd 2> /dev/null | grep -q "nandboot"; then
    echo 'NAND'
  else
    echo 'UNKNOWN'
  fi
}

set_next_boot() {
  local name="$1"
    
  if ! is_bootname "$name"; then
    echo "Unknown boot device: $name" >&2
    return 1
  fi

  case "$name" in
    NAND)
      fw_setenv bootcmd 'run nandboot'
      ;;
    SD*)
      fw_setenv bootcmd 'run mmcboot'
      local dev="$(get_root_device "$name")"
      fw_setenv mmcpart "${dev#$SDDEV}"
      ;;
    *)
      echo "Unknown boot device: $name" >&2
      return 1
      ;;
  esac
}

fi #__included_boot_funcs
